package edu.columbia.c2b2.aracne;

import java.io.BufferedReader;
import wb.data.MicroarraySet;
import wb.script.ScriptExpressionMatrixLoaderPlugin;
import wb.script.ScriptGraphLoaderPlugin;
import wb.plugins.aracne.WeightedGraph;

import java.io.File;
import java.io.PrintStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Map;
import java.util.Vector;
import java.util.HashMap;
import java.util.Random;

import org.apache.commons.cli.ParseException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import wb.data.Annotations;

/**
 * Main class to run ARACNE
 * manjunath at c2b2 dot columbia dot edu
 */
public class Aracne {
    
    static Log log = LogFactory.getLog(Aracne.class);
    public static double KERNEL_ALPHA = 0.52477;
    public static double KERNEL_BETA = -0.24;
    public static double THRESHOLD_ALPHA = 1.062;
    public static double THRESHOLD_BETA = -48.7;
    public static double THRESHOLD_GAMMA = -0.634;
    public static Parameter p;
    
    static void usage() {
        try {
            (new Parameter()).parse(new String[]{"-help"});
        } catch (ParseException e1) {
            e1.printStackTrace();
        }
    }
    
    static void findKernelWidth(int n) {
        File f = new File(p.kernel_file);
        if (f.exists()) {
            try {
                BufferedReader br = new BufferedReader(new FileReader(f));
                String line = null;
                while ((line = br.readLine()) != null) {
                    if (!line.startsWith(">")) {
                        String[] tokens = line.split("\t");
                        KERNEL_ALPHA = Double.parseDouble(tokens[0].trim());
                        KERNEL_BETA = Double.parseDouble(tokens[1].trim());
                    }
                }
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
        p.sigma = KERNEL_ALPHA * Math.pow(n, KERNEL_BETA);
    }
    
    static void findThreshold(int n) {
        File f = new File(p.threshold_file);
        if (f.exists()) {
            try {
                BufferedReader br = new BufferedReader(new FileReader(f));
                String line = null;
                while ((line = br.readLine()) != null) {
                    if (!line.startsWith(">")) {
                        String[] tokens = line.split("\t");
                        THRESHOLD_ALPHA = Double.parseDouble(tokens[0].trim());
                        THRESHOLD_BETA = Double.parseDouble(tokens[1].trim());
                        THRESHOLD_GAMMA = Double.parseDouble(tokens[2].trim());
                    }
                }
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
        p.threshold = (THRESHOLD_ALPHA - Math.log(p.pvalue)) / ((-THRESHOLD_BETA) + (-THRESHOLD_GAMMA) * n);
    }
    public static Map<String, String> geneSymbolMap;
    public static Mutual_Info mi;
    public static MarkerStats stats;
    public static MicroarraySet data;
    
    public static int run(int argc, String[] argv) {
        p = new Parameter();
        try {
            p.parse(argv);
            Parameter.checkParameter(p);
        } catch (Exception e) {
            System.out.println("Parameter failure: " + e.getMessage());
            e.printStackTrace();
            System.out.println("");
            usage();
            return -1;
        }
        
        ScriptExpressionMatrixLoaderPlugin loader = new ScriptExpressionMatrixLoaderPlugin();
        Annotations annotations;
        try {
            loader.execute(new File(p.infile));
            data = loader.getMicroarraySet();
            annotations = loader.getAnnotations();
            geneSymbolMap = annotations.getAnnotationValues("Annotation");
        } catch (Exception e) {
            e.printStackTrace();
            return -1;
        }
        
        stats = new MarkerStats();
        stats.computeMarkerVariance(data);
        stats.computeMarkerBandwidth(data);
        stats.computeMarkerRanks(data);
        
        if (p.algorithm.equals(Parameter.ALGORITHM.FIXED_BANDWIDTH) || p.algorithm.equals(Parameter.ALGORITHM.ADAPTIVE_PARTITIONING)) {
            Util.addNoise(data);
        }
        
        if (p.mode.equals(Parameter.MODE.PREPROCESSING)) {
            Thresholds.generateKernelWidthConfiguration(p.getAlgorithm());
            findKernelWidth(data.size());
            mi = new Mutual_Info(data.size(), p.miSteps, p.sigma, p.getAlgorithm());
            Thresholds.generateMutualInformationThresholdConfiguration(p.getAlgorithm());
            return 0;
        } else if (p.mode.equals(Parameter.MODE.COMPLETE)) {
            Thresholds.generateKernelWidthConfiguration(p.getAlgorithm());
            Thresholds.generateMutualInformationThresholdConfiguration(p.getAlgorithm());
        }
        
        {
            WeightedGraph weightedGraph = run(data);
            if (weightedGraph != null) {
                return 1;
            } else {
                return -1;
            }
        }
    }
    
    public static WeightedGraph run(MicroarraySet data) {
        p.infile = data.getName();
        {
            // use resampling data
            if (p.sample > 0) {
                System.out.println("Resampling data");
                Util.bootStrap(data);
            }
            
            // read the input dataset
            Matrix matrix = new Matrix();
            MarkerStats stats = new MarkerStats();
            stats.computeMarkerVariance(data);
            stats.computeMarkerBandwidth(data);
            stats.computeMarkerRanks(data);
            MatrixOp aracne = new MatrixOp(stats);
            
            if (p.mean > 0 || p.cv > 0) {
                int ndisabled = MatrixOp.filter(data, stats, p.mean, p.cv, 0);
                System.out.println(ndisabled + " markers disabled due to lack of dynamic range.");
            }
            
            Parameter.ALGORITHM TYPE = p.algorithm;
            
            Vector<Integer> ids = new Vector<Integer>();
            Map<Integer, Integer> transfac = new HashMap<Integer, Integer>();
            Vector<Integer> low = new Vector<Integer>();
            Vector<Integer> high = new Vector<Integer>();
            Vector<Integer> bs = new Vector<Integer>();
            Vector<Integer> v = null;
            int controlId = -1;
            int nsample = data.size();
            
            if (p.controlId != "") {
                controlId = data.getMarkers().getMarkerPositionByName(p.controlId);
                if (controlId == -1) {
                    System.out.println("Cannot find marker: " + p.controlId + "!");
                    return null;
                }
                boolean isHigh = (p.condition == "+") ? true : false;
                MatrixOp.getHighLowPercent(data, p.percent, controlId, low, high);
                v = (isHigh) ? high : low;
                nsample = v.size();
            }
            
            System.out.println("Marker No: " + data.getMarkers().size() + " (" + data.getMarkers().size() + " active)" + ", Array No: " + nsample);
            
            if (p.sigma == 99) {
                findKernelWidth(nsample);
                System.out.println("Kernel width determined for this dataset: " + p.sigma);
            }
            
            if (p.threshold == 0 && p.pvalue != 1) {
                findThreshold(nsample);
                System.out.println("MI threshold determined for p=" + p.pvalue + ": " + p.threshold);
            }
            
            mi = new Mutual_Info(nsample, p.miSteps, p.sigma, TYPE);
                       
            // the parseParameter function should have made sure that p.hub and p.subnet
            // can not be supplied at the same time
            if (!p.hub.equals("")) {
                int hubId = data.getMarkers().getMarkerPositionByName(p.hub);
                if (hubId == -1) {
                    System.err.println("Cannot find the hub probe: " + p.hub + ", nothing to be computed!");
                    return null;
                }
                ids.add(hubId);
            }
            
            // Subnets
            if (!"".equals(p.subnetfile)) {
                int cnt = Parameter.readProbeList(p.subnetfile, p.subnet);
            }
            if (!(p.subnet.isEmpty())) {
                for (int i = 0; i < p.subnet.size(); i++) {
                    int gid = data.getMarkers().getMarkerPositionByName(p.subnet.get(i));
                    log.debug("Adding probe " + p.subnet.get(i) + " to subnet list.");
                    if (gid == -1) {
                        System.out.println("Cannot find probe: " + p.subnet.get(i) + " in \"" + p.subnetfile + "\" ... ignored.");
                    } else {
                        ids.add(gid);
                    }
                }
            }
            
            //  Transfacs / annots
            if (!"".equals(p.annotfile)) {
                int cnt = Parameter.readProbeList(p.annotfile, p.tf_list);
            }
            if (!(p.tf_list.isEmpty())) {
                for (int i = 0; i < p.tf_list.size(); i++) {
                    int gid = data.getMarkers().getMarkerPositionByName(p.tf_list.get(i));
                    if (gid == -1) {
                        System.out.println("Cannot find probe: " + p.tf_list.get(i) + " in \"" + p.annotfile + "\" ... ignored.");
                    } else {
                        transfac.put(gid, 1);
                    }
                }
            }
            
            if (p.outfile == "") {
                p.createOutfileName(p);
            }
            
            Parameter.displayParameter(p);
            // if existing adjacency matrix is provided, no MI computation needed
            if (p.adjfile != "" || p.getPrecomputedAdjacencies() != null) {
                if (p.getPrecomputedAdjacencies() == null) {
                    // Reading adjacency matrix
                    ScriptGraphLoaderPlugin loader = new ScriptGraphLoaderPlugin();
                    try {
                        loader.execute(new File(p.adjfile), Double.MIN_VALUE);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    matrix = new Matrix(data, loader.getWeightedGraph());
                } else {
                    matrix = new Matrix(data, p.getPrecomputedAdjacencies());
                }
                
                if (p.eps != 1) {
                    System.out.println("[NETWORK] Applying DPI ...");
                    aracne.reduce(data.getMarkers(), data, matrix, mi, p.eps, p.threshold, p.correction, ids, v, transfac);
                }
            }
            
            // this is THE function that reconstruct the network!
            // 1) if conditional network is being reconstructed, 'controlId' will be the
            // gene id of the conditioning gene, and 'v' points to the set of array ids
            // in which the conditioning gene is certain percentage high/low.
            // 2) if 'ids' not empty, a subset of the network will be reconstructed
            // 3) if 'v' is non-empty while 'controlId' is -1, then bootstrapping is being performed.
            // 4) if MI correction is taken place, 'p.correction' will be a non-zero real number
            aracne.createEdgeMatrix(data.getMarkers(), data, mi, matrix, p.threshold, controlId, p.correction, ids, v);
            
            if (p.eps != 1) {
                System.out.println("[NETWORK] Applying DPI ...");
                aracne.reduce(data.getMarkers(), data, matrix, mi, p.eps, p.threshold, p.correction, ids, v, transfac);
            }
            
            try {
                if (!p.isSuppressFileWriting()) {
                    matrix.write(new PrintStream(new File(p.outfile)), data, ids, p, true, stats);
                } else {
                    log.debug("Output file writing was suppressed, skipping file writing stage.");
                }
            } catch (FileNotFoundException e) {
                log.error(e);
            }
            return matrix.getWeightedGraph(data, true);
        }
    }
    
    public static void main(String[] args) {
        long stime = System.currentTimeMillis();
        run(args.length, args);
        System.out.println("Total runtime (sec): " + ((System.currentTimeMillis() - stime) / 1000));
    }
}
