package edu.columbia.c2b2.aracne;

import wb.data.MicroarraySet;
import wb.data.Marker;
import wb.data.MarkerSet;
import wb.plugins.aracne.WeightedGraph;
import wb.plugins.aracne.GraphEdge;

import java.io.PrintStream;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

/**
 * Converted from Matrix.cpp and Matrix.h in original ARACNE source.
 * User: mhall
 * Date: Aug 21, 2006
 * User: manjunath at c2b2 dot columbia dot edu
 * Date: Nov 15, 2008
 * Time: 1:53:59 PM
 */
public class Matrix extends Vector<HashMap<Integer, Node>> {
    boolean writeTriangular;
    boolean writeReduced;
    boolean writeEmptyGenes;

    ArrayList<String> nodes = new ArrayList<String>();

    public Matrix() {
        writeTriangular = false;  // if true, only triangular half of the adjacency matrix will be written
        writeReduced = false;     // if true, intermediate nodes will be written
        writeEmptyGenes = false;  // if true, empty lines will be written
    }

    /**
     * Load the graph and populate the Matrix in the natural order of the graph.
     * @param graph
     */
    public Matrix(WeightedGraph graph) {
        this();
        nodes = new ArrayList<String>(graph.getNodes());
        for (String nodename : nodes) {
            HashMap<Integer, Node> edgeMap = new HashMap<Integer, Node>();
            add(edgeMap);
            List<GraphEdge> edgesForNode = graph.getEdgesForNode(nodename);
            for (GraphEdge graphEdge : edgesForNode) {
                Node node = new Node(graphEdge.getWeight());
                if (graphEdge.getNode1().equals(nodename)) {
                    edgeMap.put(nodes.indexOf(graphEdge.getNode2()), node);
                } else {
                    edgeMap.put(nodes.indexOf(graphEdge.getNode1()), node);
                }
            }
        }
    }

    /**
     * Load the graph and populate the Matrix in the order of the arrays present in the MicroarraySet.
     * @param data
     * @param graph
     */
    public Matrix(MicroarraySet data, WeightedGraph graph) {
        this();
        nodes = new ArrayList<String>();
        MarkerSet markers = data.getMarkers();
        for (int i = 0; i < markers.size(); i++) {
            String marker = markers.getMarker(i).getName();
            nodes.add(marker);
        }
        for (String nodename : nodes) {
            HashMap<Integer, Node> edgeMap = new HashMap<Integer, Node>();
            add(edgeMap);
            List<GraphEdge> edgesForNode = graph.getEdgesForNode(nodename);
            for (GraphEdge graphEdge : edgesForNode) {
                Node node = new Node(graphEdge.getWeight());
                if (graphEdge.getNode1().equals(nodename)) {
                    edgeMap.put(nodes.indexOf(graphEdge.getNode2()), node);
                } else {
                    edgeMap.put(nodes.indexOf(graphEdge.getNode1()), node);
                }
            }
        }
    }

    /**
     * Convert to WeightedGraph using the gene names previously set when constructing using a WeightedSet
     *
     * @return
     */
    public WeightedGraph getWeightedGraph(boolean reduced) {
        WeightedGraph graph = new WeightedGraph("Matrix Output");
        int count = 0;
        for (String nodename : nodes) {
            HashMap<Integer, Node> hashMap = get(count);
            for (Map.Entry<Integer, Node> entry : hashMap.entrySet()) {
                int intermed = entry.getValue().Get_Intermediate();
                if (intermed == 0 || !reduced) {
                    graph.addNode(nodename);
                    graph.addNode(nodes.get(entry.getKey()));
                    graph.addEdge(nodename, nodes.get(entry.getKey()), (float) entry.getValue().Get_Mutualinfo());
                }
            }
            count++;
        }
        return graph;
    }

    /**
     * Convert using the gene names in the passed in MicroarraySet to populate the nodes in the WeightedGraph
     *
     * @param data
     * @return
     */
    public WeightedGraph getWeightedGraph(MicroarraySet data, boolean reduced) {
        WeightedGraph graph = new WeightedGraph("Matrix Output");
        int count = 0;
        for (HashMap<Integer, Node> hashMap : this) {
            if (hashMap.size() > 0) {
                String nodename = data.getMarkers().getMarker(count).getName();
                for (Map.Entry<Integer, Node> entry : hashMap.entrySet()) {
                    int intermed = entry.getValue().Get_Intermediate();
                    if (intermed < 0 || !reduced) {
                        graph.addNode(nodename);
                        graph.addNode(data.getMarkers().getMarker(entry.getKey()).getName());
                        graph.addEdge(nodename, data.getMarkers().getMarker(entry.getKey()).getName(), (float) entry.getValue().Get_Mutualinfo());
                    }
                }
            }
            count++;
        }
        return graph;
    }

//
// writes an adjacency matrix to an output stream. The format is
//

    void write(PrintStream out, MicroarraySet data, Vector<Integer> ids, Parameter p, boolean writeFull, MarkerStats stats) {
        if (writeFull) {
//        std::fstream output( p.outfile.c_str(), std::ios::out );
            System.out.println("Writing matrix: " + p.outfile);

            out.println(">  Input file      " + p.infile);
            out.println(">  ADJ file        " + p.adjfile);
            out.println(">  Output file     " + p.outfile);
            out.println(">  Algorithm       " + p.algorithm);
            out.println(">  Kernel width    " + p.sigma);
            out.println(">  No. bins        " + p.miSteps);
            out.println(">  MI threshold    " + p.threshold);
            out.println(">  MI P-value      " + p.pvalue);
            out.println(">  DPI tolerance   " + p.eps);
            out.println(">  Correction      " + p.correction);
            out.println(">  Subnetwork file " + p.subnetfile);
            out.println(">  Hub probe       " + p.hub);
            out.println(">  Control probe   " + p.controlId);
            out.println(">  Condition       " + p.condition);
            out.println(">  Percentage      " + p.percent);
            out.println(">  TF annotation   " + p.annotfile);
            out.println(">  Filter mean     " + p.mean);
            out.println(">  Filter CV       " + p.cv);

            write(out, data, ids, stats);
            out.flush();
            out.close();
        }
    }


    void write(PrintStream out, MicroarraySet data, Vector<Integer> ids, MarkerStats stats) {
        if (ids.isEmpty()) {
            for (int i = 0; i < size(); i++) {
                writeGeneLine(out, data, i, stats);
            }
            out.flush();
        } else {
            for (int i = 0; i < ids.size(); i++) {
                writeGeneLine(out, data, ids.get(i), stats);
            }
        }
    }


    void writeGeneLine(PrintStream out, MicroarraySet data, int Id, MarkerStats stats) {
        DecimalFormat format = new DecimalFormat("##.########");
        HashMap<Integer, Node> m = get(Id);
//        hash_map<int, Node> & m = ( * this ) [Id];
        Marker marker = data.getMarkers().getMarker(Id);
        String label = marker.getName();
//        String label = marker.Get_Accnum();

        if ((m.size() == 0) && (!writeEmptyGenes || (!stats.getMarkerStat(marker).isEnabled()))) {
//        if ((m.size() == 0) && (!writeEmptyGenes || (!stats.getMarkerStat(marker).isEnabled() && !marker.Is_Control()))) {
            return;
        }

        out.print(label);
//        out + label;

        Vector<Integer> sorted_keys = new Vector<Integer>();
        for (Map.Entry<Integer, Node> entry : m.entrySet()) {
            sorted_keys.add(entry.getKey());
        }
//        for (const_line_iterator it = m.begin(); it != m.end(); ++it) {
//            sorted_keys.push_back(it - > first);
//        }
        Collections.sort(sorted_keys);

        for (Integer viter : sorted_keys) {

//        Vector<Integer>::const_iterator viter;
//        for (viter = sorted_keys.begin(); viter != sorted_keys.end(); ++viter) {
            int Gene_Id = viter;
            Node node = m.get(viter);
            if (writeTriangular && Gene_Id <= Id)
                continue;

            Marker mx = data.getMarkers().getMarker(Gene_Id);
            String affyid = mx.getName();
            if (writeReduced) {
                out.print('\t' + affyid);
//                out << '\t' << affyid;
                if (node.Get_Intermediate() >= 0) {
                    out.print('.' + node.Get_Intermediate());
//                    cout << '.' << node.Get_Intermediate();
                }
                out.print('\t' + node.Get_Mutualinfo());
//                out << '\t' << node.Get_Mutualinfo();
            } else {
                if (node.Get_Intermediate() >= 0)
                    continue;
                double mi = node.Get_Mutualinfo();
                out.print('\t' + affyid + '\t' + format.format(mi));
//                out << '\t' << affyid << "\t" << mi;
            }
        }
        out.println();
//        out << endl;
    }

    void writeGeneList(PrintStream out, MicroarraySet data, String name, int probeId, MarkerStats stats) {
        String filename = name + ".adj";
//  std::fstream output( filename.c_str(), std::ios::out );
        System.out.println("Writing gene list: " + filename);

        HashMap<Integer, Node> m = get(probeId);
//    Hashmap< Integer, Node > & m = ( * this ) [probeId];
        Marker marker = data.getMarkers().getMarker(probeId);

        if ((m.size() == 0) && (!writeEmptyGenes || (!stats.getMarkerStat(marker).isEnabled() && !stats.getMarkerStat(marker).isControl())))
        {
            return;
        }


        for (Map.Entry<Integer, Node> entry : m.entrySet()) {
            if (entry.getKey() != probeId) {
                out.println(entry.getKey() + '\t' + data.getMarkers().getMarker(entry.getKey()).getName() + '\t' + entry.getValue().Get_Mutualinfo());
//            output << j << "\t" << data.Get_Marker( j ).Get_Accnum() << "\t" << n.Get_Mutualinfo() << endl;
            }
        }
        /*
            hash_map < int, Node >::const_iterator it = m.begin();
          while ( it != m.end() )
          {
            Node const & n = it->second;
            int j = it->first;
            if ( j != probeId )
            {
              output << j << "\t" << data.Get_Marker( j ).Get_Accnum() << "\t" << n.Get_Mutualinfo() << endl;
            }
            it++;
          }

          output.flush();
          output.close();
        */
    }

    boolean hasNode(int i, int j) {
        int ii = Math.max(i, j);
        int jj = Math.min(i, j);
        if (ii == jj)
            return false;
        if (size() <= ii)
            return (false);
        HashMap<Integer, Node> m = get(i);
        if (m.get(j) == null)
            return (false);
        return (true);
    }


    void setWriteTriangular(boolean status) {
        writeTriangular = status;
    }

    void setWriteReduced(boolean status) {
        writeReduced = status;
    }

    void setWriteEmptyGenes(boolean status) {
        writeEmptyGenes = status;
    }
}
