package edu.columbia.c2b2.aracne;

import wb.data.MicroarraySet;
import wb.data.MarkerSet;
import wb.data.Marker;

import edu.columbia.c2b2.aracne.Util.Gene;
import edu.columbia.c2b2.aracne.Util.Gene_Pair;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author manjunath at c2b2 dot columbia dot edu
 */

public class MatrixOp {

    static Log log = LogFactory.getLog(MatrixOp.class);

    MarkerStats stats = new MarkerStats();

    public MatrixOp(MarkerStats stats) {
        this.stats = stats;
    }


    /* Create entries for all the rows in the matrix */
    void createEntries(Matrix matrix, int entry) {
        for (int i = 0; i < entry; i++) {
            matrix.add(new HashMap<Integer, Node>());
        }
    }

    /* Add a node to the matrix */
    void addNode(Matrix matrix, int i, int j, double edgeValue, boolean symmetric) {
        Node node = new Node(0.0);
        node.Set_Mutualinfo(edgeValue);
        matrix.get(i).put(j, node);
        if (symmetric) {
            matrix.get(j).put(i, node);
        }
    }

    /**
     * Operation stop request
     */
    boolean stopped = false;
    /* * The percent completed */
    double completion = 0.0;

    public static final double NO_CONNECTION = 0.0;
    HashMap<Integer, Node> const_line_iterator;
    int uint;

    void createEdgeMatrix(MarkerSet markers, MicroarraySet arrayData, Mutual_Info mutual, Matrix matrix,
                          double threshold, int controlId, double noise2, Vector<Integer> ids, Vector<Integer> v) {
        long t1, t2;

        t1 = System.currentTimeMillis();

        int markerNo = markers.size();
        if (ids.isEmpty()) {
            int step = (int) Math.ceil(0.1 * markerNo);
            createEntries(matrix, markerNo);
            // Loop over all the probes
            for (int i = 0; i < markerNo; i++) {
                if ((i != controlId) && (isMarkerEnabled(markers, i))) {
                    // Process only markers that are active (i.e. ones that are not filtered out)
                    computeOneRow(markers, mutual, matrix, threshold, i, markerNo, controlId, v, true, true, noise2, arrayData);
                }
                if (((i + 1) % step) == 0) {
                    t2 = System.currentTimeMillis();
                    System.out.println((10 * (i + 1) / step) + "%, time: " + (t2 - t1));
                }
            }
            t2 = System.currentTimeMillis();
            System.out.println("Gene: " + markerNo + " Time: " + (t2 - t1));
        } else {
            int step = (int) Math.ceil(0.1 * ids.size());
            for (int i = 0; i < ids.size(); i++) {
                if (ids.get(i) == controlId) {
                    continue;
                }

                while (matrix.size() <= (int) ids.get(i)) {
                    matrix.add(new HashMap<Integer, Node>());
                }
                computeOneRow(markers, mutual, matrix, threshold, ids.get(i), markerNo, controlId, v, false, false, noise2, arrayData);
                if (((i + 1) % step) == 0) {
                    t2 = System.currentTimeMillis();
                    System.out.println((10 * (i + 1) / step) + "%, time: " + (t2 - t1));
                }
            }
            t2 = System.currentTimeMillis();
            System.out.println("Gene: " + markerNo + " Time: " + (t2 - t1));
        }
    }

    void computeOneRow(MarkerSet markers, Mutual_Info mutual, Matrix matrix, double threshold, int row_idx,
                       int markerNo, int controlId, Vector<Integer> v, boolean half_matrix, boolean symmetric, double noise2, MicroarraySet arrayData) {
        int j = 0;
        if (half_matrix) {
            j = row_idx + 1;
        }
        
        for (; j < markerNo; j++) {
            if ((j != controlId) && (isMarkerEnabled(markers, j))) {
                double edge = calculateMI(arrayData, mutual, row_idx, j, threshold, noise2, v);
                if (edge != NO_CONNECTION) {
                    addNode(matrix, row_idx, j, edge, symmetric);
                }
            }
        }
    }

 /* compute mutual information between two gene expression vectors */
    double calculateMI(MicroarraySet data, Mutual_Info mutual, int probeId1, int probeId2, double threshold, double noise2, Vector<Integer> v ) {
        double edge = MatrixOp.NO_CONNECTION;
        if ( isSameGene(data.getMarkers(), probeId1, probeId2)) {
            return edge;
        }
                
        double sigmaX = stats.getMarkerStat(data.getMarkers().getMarker(probeId1)).getBandwidth();
        double sigmaY = stats.getMarkerStat(data.getMarkers().getMarker(probeId2)).getBandwidth();
        
        Vector<Gene> g1 = stats.getMarkerStat(data.getMarkers().getMarker(probeId1)).getRankedValues();
        Vector<Gene> g2 = stats.getMarkerStat(data.getMarkers().getMarker(probeId2)).getRankedValues();
        
        double mi = mutual.Compute_Pairwise_MI(g1, g2, sigmaX, sigmaY);
        if (mi >= threshold) {
            edge = mi;
            // correct for bias due to array measurement noise
            if (noise2 > 0) {
                double v1 = stats.getMarkerStat(data.getMarkers().getMarker(probeId1)).getVariance();
                double v2 = stats.getMarkerStat(data.getMarkers().getMarker(probeId2)).getVariance();
                double lambda = (v1/(v1-noise2))*(v2/(v2-noise2));
                edge = mi + 0.5 * Math.log(1 + (Math.exp((2*mi)-1)*(1-1/lambda)));
            }
        }
        return edge;
    }
    
    /**
     * removes non contiguous edges by using the triangle inequality
     * markers       - microarray dataset
     * mutual     - mutual information calculator
     * matrix     - the edge matrix
     * threshld   - mutual informaiton threshold
     * epsilon    - DPI tolerance
     * ids        - vector of gene id for which edge matrix will be computed; if ids
     * is empty, all genes will be computed
     * v          - a pointer to the vector which contains the array ids used for
     * mutual information computation
     * constrained- a boolean to determine whether new MI need to be computed. For '-l'
     * option, we don't want to compute any new MI other than those already
     * computed for probes provided. For '-h' option, we first compute
     * only the row in the adjacency matrix corresponding to the probe of
     * interest; if DPI need to be performed, we will further compute more
     * MIs as triplets can be formed.
     */
    void reduce(MarkerSet markers, MicroarraySet arrayData, Matrix matrix, Mutual_Info mutual, double epsilon, double threshold,
                double noise2, Vector<Integer> ids, Vector<Integer> v, Map<Integer, Integer> transfac) {
        long t1; // start time
        long t2; // end time
        t1 = System.currentTimeMillis();
        if (ids.isEmpty()) {
            // apply DPI on all the nodes
            int markerNo = matrix.size();
            for (int i = 0; i < markerNo; i++) {
                reduceOneNode(i, markers, matrix, mutual, epsilon, threshold, noise2, v, transfac, arrayData);
            }
        } else {
            for (int i = 0; i < ids.size(); i++) {
                reduceOneNode(ids.get(i), markers, matrix, mutual, epsilon, threshold, noise2, v, transfac, arrayData);
            }
        }
        t2 = System.currentTimeMillis();
        System.out.println("DPI running time is: " + (t2 - t1));
    }

    void reduceOneNode(int row_idx, MarkerSet markers, Matrix matrix,
                       Mutual_Info mutual, double epsilon, double threshold, double noise2, Vector<Integer> v, Map<Integer, Integer> transfac,
                       MicroarraySet arrayData) {
        HashMap<Integer, Node> m = matrix.get(row_idx);
        Vector<ArrayValuePair> miVector = new Vector<ArrayValuePair>();
        // put all the values of the i-th row in the matrix on a vector for sorting
        // the ArrayValuePair structure is borrowed from the Microarray_Set class,
        // it should actually be called EdgeMIPair here. They share the same
        // structure consisting of two fields: a int and a double. The assoiciated
        // sorting class will sort by the double fields.
        for (Map.Entry<Integer, Node> entry : m.entrySet()) {
             miVector.add(new ArrayValuePair(entry.getKey(), entry.getValue().Get_Mutualinfo()));
        }

        // sort the vector
        Collections.sort(miVector, new SortDecreasing_ArrayValuePair());

        // for each value in the vector (these are genes directly connected to row_idx.
        // i.e. geneId1 is connected to i)
        for (int j = 0; j < miVector.size(); j++) {
            // (geneId1, valueAB) are the (id, value) of the selected gene, starting with the largest mi
            int geneId1 = miVector.get(j).getId();
            double valueAB = miVector.get(j).getValue();

            // Set the limits
            double minMI = valueAB / (1.0 - epsilon);
            
            // Loop on all the genes that have an equal or higher mutual information
            // (these genes are also connected to i) bool connected = true;
            for (int k = 0; k < j; k++) {
                int geneId2 = miVector.get(k).getId();

                double valueAC = miVector.get(k).getValue();
                // We know that valueAB is smaller than valueAC by default; then tolerance is always positive
                if (valueAC <= minMI) {
                    // we can exit from this loop because all future values are smaller
                    break;
                }
                // We know that valueAB is smaller than valueAC by default.
                // Therefore, if it is also smaller than valueBC, then
                // geneId1 is connected to gene i via geneId2
                double valueBC = getNodeMI(matrix, geneId1, geneId2);
                if (valueBC > minMI) {
                    // if TF annotation information is provided, the triangle will be
                    // broken only if certein logic is met
                    if (!(transfac.isEmpty())) {
                        if (protectedByTFLogic(transfac, row_idx, geneId1, geneId2)) {
                            continue;
                        }
                    }

                    Node node = m.get(geneId1);
                    node.Set_Intermediate(geneId2);
                    break;
                }
            }
        }
    }

   /**
    * fill the pair vector with gene expression values for MI computation. For the
    * unconditional MI calculation (array = NULL), expresssion values from all
    * arrays are used; for the conditional case (array not NULL), only microarrays * in the array vector are used.
    *
    */
    void fillPair(int maNum, MicroarraySet matrix, int ID1, int ID2, Vector<Gene_Pair> pv, Vector<Integer> arrays ) {
        if (arrays != null) {
            for(int i = 0; i < maNum; i++ ) {
                Gene_Pair pair = new Gene_Pair();
                Set_Pair( pair, matrix.getValue(arrays.get(i), ID1 ), matrix.getValue(arrays.get(i), ID2 ), i);
                pv.add(pair);
            }
        } else {
            for (int i = 0; i < maNum; i++ ) {
                Gene_Pair pair = new Gene_Pair();
                Set_Pair( pair, matrix.getValue( i, ID1 ), matrix.getValue( i, ID2 ), i );
                pv.add(pair);
            }
        }
    }
    
    void setYValue(int maNum, float[][] matrix, int ID1, int ID2, Vector<Gene_Pair> pv, Vector<Integer> arrays) {
        if (arrays != null) {
            for (int i = 0; i < maNum; i++) {
                Gene_Pair pair = pv.get(i);
                pair.y = matrix[arrays.get(pair.maId)][ID2];
            }
        } else {
            for (int i = 0; i < maNum; i++) {
                Gene_Pair pair = pv.get(i);
                pair.y = matrix[pair.maId][ID2];
            }
        }
    }

    void Set_Pair(Gene_Pair p, double xId, double yId, int maId) {
        p.x = xId;
        p.y = yId;
        p.maId = maId;
    }


    boolean isMarkerEnabled(MarkerSet markers, int i) {
        return stats.getMarkerStat(markers.getMarker(i)).isEnabled();
    }


    boolean isSameGene(MarkerSet markers, int i, int j) {
        boolean nameComparison = false;
        if (Aracne.geneSymbolMap != null){
            nameComparison = ((Aracne.geneSymbolMap.get(markers.getMarker(i).getName()).equals(Aracne.geneSymbolMap.get(markers.getMarker(j).getName()))) &&
                        (!Aracne.geneSymbolMap.get(markers.getMarker(i).getName()).equals("---")));
        }
        return ((i == j) || nameComparison);
    }

/* the return of this function can be 3 cases:
 * case I: a postive real number - the node has been computed and survived the
           thresholding, so the MI value is returned
   case II: 0 - the node has been computed but did not survive the thresholding
   case III: -1 - the node has not yet been computed
 */

    double getNodeMI(Matrix matrix, int geneId1, int geneId2) {
        double mi = -1;
        if (matrix.size() > geneId1) {
            HashMap<Integer, Node> m12 = matrix.get(geneId1);
            if (!(m12.isEmpty())) {
                // m12 can be empty for two reasons:
                // 1) it was simply pushed onto the hash as it is on the way for other rows
                //    to be computed
                // 2) it was actually computed, but not a single MI estimation pass the threshold.
                //    this case is possible but less likely to happen
                if (m12.get(geneId2) != null) {
                    Node n12 = m12.get(geneId2);
                    mi = n12.Get_Mutualinfo();
                } else {
                    // now that there are other entries on this row that have been computed,
                    // this entry being absent is due to the fact that it is less than the threshold.
                    // in this case, it obviously can not break the triangle
                    mi = 0;
                }
            }
        }

        // for full network reconstruction, since the computations were symmetric, knowning
        // that n12 does not exist or has been thresholded will be sufficient.
        // however, for cases where we only compute a portion of the full adjacency matrix,
        // we have to check n21 to make sure.
        if ((mi == -1) && (matrix.size() > geneId2)) {
            HashMap<Integer, Node> m21 = matrix.get(geneId2);
            if (!(m21.isEmpty())) {
                if (m21.get(geneId1) != null) {
                    Node n21 = m21.get(geneId1);
                    mi = n21.Get_Mutualinfo();
                } else {
                    mi = 0;
                }
            }
        }
        return mi;
    }

    boolean protectedByTFLogic(Map<Integer, Integer> transfac, int geneId1, int geneId2, int geneId3) {
        boolean isA = (transfac.get(geneId1) != null);
        boolean isB = (transfac.get(geneId2) != null);
        boolean isC = (transfac.get(geneId3) != null);
        
        if ( isC ) {
            return false;
        } else if (!(isA || isB)) {
            return false;
        }
        return true;
    }

    static class ArrayValuePair {
        int arrayId;
        double value;

        ArrayValuePair(int Id, double v) {
            set(Id, v);
        }

        void set(int iId, double v) {
            arrayId = iId;
            value = v;
        }

        int getId() {
            return arrayId;
        }

        double getValue() {
            return value;
        }
        
        @Override
        public boolean equals(Object o) {
            ArrayValuePair avp = (ArrayValuePair)o;
            return (avp.arrayId == arrayId && avp.value == value);
        }

        @Override
        public int hashCode() {
            int hash = 5;
            hash = 71 * hash + this.arrayId;
            hash = 71 * hash + (int) (Double.doubleToLongBits(this.value) ^ (Double.doubleToLongBits(this.value) >>> 32));
            return hash;
        }
    }

    static class SortIncreasing_ArrayValuePair implements Comparator<ArrayValuePair> {
        public int compare(ArrayValuePair o1, ArrayValuePair o2) {
            return (int) Math.signum(o1.getValue() - o2.getValue());
        }
    }

    static class SortDecreasing_ArrayValuePair implements Comparator<ArrayValuePair> {
        public int compare(ArrayValuePair o1, ArrayValuePair o2) {
            return (int) Math.signum(o2.getValue() - o1.getValue());
        }
    }

    public static void getHighLowPercent(MicroarraySet set, double x, int mId, Vector<Integer> lower, Vector<Integer> upper) {
        Vector<ArrayValuePair> sortArray = new Vector<ArrayValuePair>();
        MatrixOp.SortIncreasing_ArrayValuePair sorter = new MatrixOp.SortIncreasing_ArrayValuePair();
        int idNo = set.size();
        for (int iId = 0; iId < idNo; iId++) {
            MatrixOp.ArrayValuePair pair = new ArrayValuePair(iId, set.getValue(iId, mId));
            sortArray.add(pair);
        }
        Collections.sort(sortArray, sorter);
        int idPercNo = (int) ((double) idNo * x);
        for (int iId = 0; iId < idPercNo; iId++) {
            lower.add(sortArray.get(iId).getId());
            upper.add(sortArray.get(idNo - idPercNo - 1 + iId).getId());
        }
    }

    /**
     * filters the Microarray_set by disabling all the markers whose mean
     * is smaller than minMean and standard deviation is smaller than the mean * ratio
     */
    private static int filter(MicroarraySet set, MarkerStats stats, Vector<Integer> ids, double min_mean, double min_sigma, int ctlid) {
        double iNo = (double) set.size();
        MarkerSet markers = set.getMarkers();
        int ndisabled = markers.size();

        for (int i = 0; i < ndisabled; i++) {
            Marker marker = markers.getMarker(i);
            stats.getMarkerStat(marker).setEnabled(false);
        }
        for (int m = 0; m < markers.size(); m++) {
            if ((int) m == ctlid)
                continue;
            double nx = 0;
            double nxx = 0;
            for (Integer i : ids) {
                double v = set.getValue(i, m);
                nx += v;
                nxx += v * v;
            }

            double mean = nx / iNo;
            double stdev = Math.sqrt((iNo * nxx - nx * nx) / (iNo * iNo));
            log.debug(m + " mean: " + mean + ", stdev: " + stdev);
            if ((mean >= min_mean) && (stdev >= mean * min_sigma)) {
                stats.getMarkerStat(markers.getMarker(m)).setEnabled(true);
                ndisabled--;
            }
        }
        return (ndisabled);
    }

//
// This version generates the control set.
//
    public static int filter(MicroarraySet set, MarkerStats stats, double min_mean, double min_sigma, int ctlid) {
        Vector<Integer> v = new Vector<Integer>();
        int i;
        int r;

        for (i = 0; i < set.size(); i++)
            v.add(i);
        r = filter(set, stats, v, min_mean, min_sigma, ctlid);
        return(r);
    }
}