package edu.columbia.c2b2.aracne;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;

import java.util.Vector;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import wb.plugins.aracne.WeightedGraph;

/**
 * 
 * @author manjunath at c2b2 dot columbia dot edu
 */

public class Parameter {

   public static enum ALGORITHM {NAIVE_BAYES, FIXED_BANDWIDTH, VARIABLE_BANDWIDTH, ADAPTIVE_PARTITIONING}; 
   
   public static enum MODE {PREPROCESSING, DISCOVERY, COMPLETE};
   
// Default parameters:
// mi threshold
    public static final double default_threshold = 0.0;
// p-value for mi threshold
    public static final double default_pvalue = 1.0;
// DPI tolerance
    public static final double default_eps = 1.0;
// kernel width
    public static final double default_sigma = 99;
// mi step size
    public static final int default_miSteps = 6;
// sample number
    public static final int default_sample = 0;
// high/low percentage
    public static final double default_percent = 0.35;
// filter mean
    public static final double default_mean = 0.0;
// filter standard deviation
    public static final double default_cv = 0.0;
// array measurement noise level
    public static final double default_correction = 0.0;

    private boolean suppressFileWriting = false;

    double threshold; // mi threshold
    double pvalue;
    double eps;       // DPI tolerance
    double sigma;     // gaussian kernel width
    int miSteps;      // mi step size
    int sample;       // sample number
    double percent;   // high/low percentage for the conditional analysis
    double mean;      // filter mean
    double cv;        // filter coefficient of variance
    double correction;  // coorection for noise
    String verbose = "off";
    ALGORITHM algorithm;
    MODE mode;
    String infile = "";
    String outfile = "";
    String adjfile = "";
    WeightedGraph precomputedAdjacencies;
    String hub = "";
    String subnetfile = "";
    String annotfile = "";
    String controlId = "";
    String condition = "";
    String home_dir = "./";
    String threshold_file = "config_threshold.txt";
    String kernel_file = "config_kernel.txt";
    Vector<String> subnet = new Vector<String>();
    Vector<String> tf_list = new Vector<String>();
        
    public Parameter() {
        this.algorithm = ALGORITHM.FIXED_BANDWIDTH;
        this.mode = MODE.DISCOVERY;
        this.threshold = default_threshold;
        this.pvalue = default_pvalue;
        this.eps = default_eps;
        this.sigma = default_sigma;
        this.miSteps = default_miSteps;
        this.sample = default_sample;
        this.percent = default_percent;
        this.mean = default_mean;
        this.cv = default_cv;
        this.correction = default_correction;
    }


    String getFileName(String matrixName) {
        int len = matrixName.length();
        int b = matrixName.lastIndexOf("/");
        if (b == -1) {
            b = matrixName.lastIndexOf("\\");
        }
        String dirname = "";
        String basename = matrixName;
        if ((b != -1) && (b < len)) {
            // Extract the directory and the filename if path is included
            basename = matrixName.substring(b + 1, len);
            dirname = matrixName.substring(0, b) + "/";
        }
        int c = basename.lastIndexOf(".");
        if (c != -1) {
            basename = basename.substring(0, c);
        }
        return dirname + basename;
    }

    static void checkParameter(Parameter p) throws Exception {
        if (p.infile == "") {
            throw(new Exception("No input file specified!"));
        }

        if ((p.subnetfile != "") && (p.hub != "")) {
            throw(new Exception("Either supply one hub gene by '-h' or multiple genes in a file by '-l', but not both!"));
        }

        if (!((p.condition == "+") || (p.condition == "-") || (p.condition == ""))) {
            throw(new Exception("Condition can be either '+' or '-', but no else!"));
        }

        if ((p.condition == "+") || (p.condition == "-")) {
            if (p.controlId == "") {
                throw(new Exception("Control gene ID must be specified using '-c'!"));
            }
        }

        if ((p.sigma != 99) && (p.sigma <= 0 || p.sigma >= 1)) {
            throw(new Exception("Kernel width '-k' must be within (0,1)!"));
        }

        if (p.threshold < 0) {
            throw(new Exception("MI threshold '-t' must be non-negative!"));
        }

        if (p.threshold > 0 && p.pvalue != 1) {
            System.out.println("P-value will not be used, since a threshold has been specified.");
        }

        if (p.pvalue <= 0 || p.pvalue > 1) {
            throw(new Exception("P-value '-p' must be in the range (0,1]!"));
        }

        if (p.eps < 0 || p.eps > 1) {
            throw(new Exception("DPI tolerance '-e' must be within [0,1]!"));
        }

        if (p.percent <= 0 || p.percent >= 1) {
            throw(new Exception("Percentage microarray must be within (0,1)!"));
        }

        if (p.miSteps <= 0) {
            throw(new Exception("Number of bins for fast method '-b' must be postive!"));
        }

        if (p.mean < 0) {
            throw(new Exception("Gene filter mean must be non-negative!"));
        }

        if (p.cv < 0) {
            throw(new Exception("Gene filter cv (coefficient of variance) must be non-negative!"));
        }

        if (p.correction < 0) {
            throw(new Exception("Array measurement noise level '-n' must be non-negative!"));
        }

        if (!(p.verbose.toLowerCase().equals("on") || (p.verbose.toLowerCase().equals("off")))) {
            throw(new Exception("Verbose '-v' can be either 'on' or 'off', but no else!"));
        }

        if (!((p.algorithm.equals(ALGORITHM.NAIVE_BAYES)) || p.algorithm.equals(ALGORITHM.FIXED_BANDWIDTH) || p.algorithm.equals(ALGORITHM.VARIABLE_BANDWIDTH) || p.algorithm.equals(ALGORITHM.ADAPTIVE_PARTITIONING))) {
            throw(new Exception("Supported algorithm '-a' is one of 'naive_bayes | nb', 'fixed_bandwidth | fb', 'variable_bandwidth | vb' or 'adaptive_partitioning | ap' alone!"));
        }

        if (!((p.mode.equals(MODE.PREPROCESSING)) || p.mode.equals(MODE.DISCOVERY)) || p.mode.equals(MODE.COMPLETE)) {
            throw(new Exception("Supported mode '-m' is one of 'preprocessing | pp', 'discovery | dx' or 'complete | ct' alone!"));
        }
        
        if (!p.home_dir.equals("./")) {
            int len = p.home_dir.length();
            int b = p.home_dir.lastIndexOf("/");
            if (b == -1 || b != (len - 1)) {
                p.home_dir = p.home_dir + "/";
            }
        }
    }

    void createOutfileName(Parameter p) {
        String filename = getFileName(p.infile);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PrintStream out = new PrintStream(baos);
        if (!p.hub.equals("")) {
            filename = filename + "_h" + p.hub;
        }
        if (!p.controlId.equals("")) {

            out.print((p.condition == "+") ? "H" : "L");
            filename = filename + "_c" + p.controlId + out.toString();
        }
        out.printf("_k%4.3g", p.sigma);
        String kernel = "%0.3g" + p.sigma;

        if (p.threshold > 0) {
            out.printf("_t%3.2g", p.threshold);
        }

        if (p.eps < 1) {
            out.printf("_e%3.2g", p.eps);
        }

        if (p.sample > 0) {
            out.printf("_r%03d", p.sample);
        }
        filename = filename + baos.toString() + ".adj";
        p.outfile = filename;
    }

    static void displayParameter(Parameter p) {
        System.out.println(p.getParamterDescription());
    }

    public String getParamterDescription() {
        StringBuilder builder = new StringBuilder();
        builder.append("");

        builder.append("[PARA] Input file:    " + infile + "\n");
        builder.append("[PARA] Output file:   " + outfile + "\n");
        if (algorithm.equals(ALGORITHM.FIXED_BANDWIDTH)) {
            builder.append("[PARA] Algorithm:     " + algorithm + "\n");
            builder.append("[PARA] Mode:     " + mode + "\n");
            if (sigma == 99) {
                builder.append("[PARA] Kernel width:  determined by program");
            } else {
                builder.append("[PARA] Kernel width:  " + sigma + "\n");
            }
        } else {
            builder.append("[PARA] Algorithm:     " + algorithm + "\n");
            builder.append("[PARA] Mode:     " + mode + "\n");
            builder.append("[PARA] No. bins:      " + miSteps + "\n");
        }
        if (threshold > 0) {
            builder.append("[PARA] MI threshold:  " + threshold + "\n");
            if (pvalue > 0) {
                builder.append("[PARA] MI threshold calculated from P-Value:  " + pvalue + "\n");
            }
        } else {
            builder.append("[PARA] MI P-value:    " + pvalue + "\n");
        }

        builder.append("[PARA] DPI tolerance: " + eps + "\n");

        if (correction > 0) {
            builder.append("[PARA] Correction for MI estimation (array noise level: " + correction + ")" + "\n");
        }

        if (subnetfile != "") {

            builder.append("[PARA] Subset of probes to reconstruct: " + subnetfile + " (" + subnet.size() + ")" + "\n");
        }
        if (hub != "") {
            builder.append("[PARA] Hub probe to reconstruct: " + hub + "\n");
        }
        if (controlId != "") {
            builder.append("[PARA] Control gene:  " + controlId + "\n");
            builder.append("[PARA] Condition:     " + condition + "\n");
            builder.append("[PARA] Percentage:    " + percent + "\n");
        }
        if (annotfile != "") {
            builder.append("[PARA] TF annotation list: " + annotfile + "\n");
        }
        if (mean != 0 || cv != 0) {
            builder.append("[PARA] Filter mean:   " + mean + "\n");
            builder.append("[PARA] Filter CV:     " + cv + "\n");
        }
        return builder.toString();
    }

    public void parse(String[] args) throws ParseException {
        Options options = new Options();
        String infile = "i";
        String name = "infile";
        Option infileOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("Input gene expression profile dataset")
                .create(infile);
        String outfile = "o";
        name = "outfile";
        Option outfileOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("Output file name (optional)")
                .create(outfile);
        String algorithm = "a";
        name = "algorithm";
        Option algorithmOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("Algorithm ((fixed_bandwidth | fb) | (variable_bandwidth | vb) | (naive_bayes | nb) | (adaptive_partitioning | ap), default: fixed_bandwidth")
                .create(algorithm);
        String mode = "m";
        name = "mode";
        Option modeOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("Mode ((preprocessing | pp) | (discovery | dx) | (complete | ct)), default: discovery")
                .create(mode);
        String kernelWidth = "k";
        name = "kernel_width";
        Option kernelOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("Kernel width (fixed_bandwidth method only), default: determined by program")
                .create(kernelWidth);
        String bins = "b";
        name = "#_bins";
        Option binsOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("No. of bins (naive_bayes method only), default: 6")
                .create(bins);
        String mi = "t";
        name = "threshold";
        Option miOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("MI threshold, default: 0")
                .create(mi);
        String pvalue = "p";
        name = "p-value";
        Option pvalueOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("P-value for MI threshold (e.g. 1e-7), default: 1 [**]")
                .create(pvalue);
        String tolerance = "e";
        name = "tolerance";
        Option toleranceOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("DPI tolerance, default: 1")
                .create(tolerance);
        String hub = "h";
        name = "probeId";
        Option hubOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("Hub gene (only MI w/ hub gene will be computed), default: NONE")
                .create(hub);
        String resample = "r";
        name = "sample_number";
        Option resampleOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("Use resampling arrays")
                .create(resample);
        String subnet = "s";
        name = "subnetwork_hub_file";
        Option subnetOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("A file containing a list of probes for which a subnetwork will be constructed, default: NONE")
                .create(subnet);
        String transfac = "l";
        name = "transcription_factor_file";
        Option transfacOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("A file containing a list of probes annotated as transcription factors in the input dataset, default: NONE [***]")
                .create(transfac);
        String reconstruct = "c";
        name = "+/-probeId %";
        Option reconstructOpt = OptionBuilder.withArgName(name)
                .withLongOpt(name)
                .hasArg()
                .withDescription("Conditional network reconstruction, default: NONE [****] [format: \"+24 0.35\", \"-1973_s_at 0.4\"]")
                .create(reconstruct);
        String filter = "f";
        name = "filter";
        Option filterOpt = OptionBuilder.hasArgs(2).withArgName("mean").withArgName("cv")
                .withLongOpt(name)
                .withDescription("Gene filter by the mean and coefficient of variance (cv) of the expression values, default: mean=0, cv=0")
                .create(filter);
        String thresholdFile = "ct";
        name = "config_threshold";
        Option thresholdFileOpt = OptionBuilder.withArgName("threshold_configuration_file")
                .withLongOpt(name)
                .hasArg()
                .withDescription("Path to a file containing the pValue - MI threshold configuration. default: \"config_threshold.txt\" file in the directory \"java\" gets invoked from")
                .create(thresholdFile);
        String kernelFile = "ck";
        name = "config_kernel";
        Option kernelFileOpt = OptionBuilder.withArgName("kernel_configuration_file")
                .withLongOpt(name)
                .hasArg()
                .withDescription("Path to a file containing Gaussian Kernel width threshold configuration. default: \"config_kernel.txt\" file in the directory \"java\" gets invoked from. This file is needed for FIXED_BANDWIDTH algorithm alone")
                .create(kernelFile);

        String help = "help";
        Option helpOpt = new Option(help, false, "Display this help and exit");

        options.addOption(infileOpt);
        options.addOption(outfileOpt);
        options.addOption(algorithmOpt);
        options.addOption(modeOpt);
        options.addOption(kernelOpt);
        options.addOption(binsOpt);
        options.addOption(miOpt);
        options.addOption(pvalueOpt);
        options.addOption(toleranceOpt);
        options.addOption(hubOpt);
        options.addOption(resampleOpt);
        options.addOption(subnetOpt);
        options.addOption(transfacOpt);
        options.addOption(reconstructOpt);
        options.addOption(filterOpt);
        options.addOption(thresholdFileOpt);
        options.addOption(kernelFileOpt);

        options.addOption(helpOpt);

        CommandLineParser parser = new GnuParser();
        CommandLine line = parser.parse(options, args);

        if (line.hasOption(help)) {
            HelpFormatter formatter = new HelpFormatter();
            formatter.printHelp("aracne", options);
            System.out.println("");
            System.out.println(usageNotes);
	    System.exit(0);
        }

        if (line.hasOption(infile)) {
            this.infile = line.getOptionValue(infile);
        }
        if (line.hasOption(outfile)) {
            this.outfile = line.getOptionValue(outfile);
        }
        if (line.hasOption(algorithm)) {
            String optionValue = line.getOptionValue(algorithm);
            if (optionValue.equalsIgnoreCase("naive_bayes") || optionValue.equalsIgnoreCase("nb")) {
                this.algorithm = ALGORITHM.NAIVE_BAYES;
            }
            else if (optionValue.equalsIgnoreCase("fixed_bandwidth") || optionValue.equalsIgnoreCase("fb")) {
                this.algorithm = ALGORITHM.FIXED_BANDWIDTH;
            }
            else if (optionValue.equalsIgnoreCase("variable_bandwidth") || optionValue.equalsIgnoreCase("vb")) {
                this.algorithm = ALGORITHM.VARIABLE_BANDWIDTH;
            }
            else if (optionValue.equalsIgnoreCase("adaptive_partitioning") || optionValue.equalsIgnoreCase("ap")) {
                this.algorithm = ALGORITHM.ADAPTIVE_PARTITIONING;
            }
            else {
                HelpFormatter formatter = new HelpFormatter();
                formatter.printHelp("aracne", options);
                System.out.println("");
                System.out.println(usageNotes);
                throw new ParseException("Incorrect algorithm option.");
            }
        }
        if (line.hasOption(mode)) {
            String optionValue = line.getOptionValue(mode);
            if (optionValue.equalsIgnoreCase("preprocessing") || optionValue.equalsIgnoreCase("pp")) {
                this.mode = MODE.PREPROCESSING;
            }
            else if (optionValue.equalsIgnoreCase("discovery") || optionValue.equalsIgnoreCase("dx")) {
                this.mode = MODE.DISCOVERY;
            }
            else if (optionValue.equalsIgnoreCase("complete") || optionValue.equalsIgnoreCase("ct")) {
                this.mode = MODE.COMPLETE;
            }
            else {
                HelpFormatter formatter = new HelpFormatter();
                formatter.printHelp("aracne", options);
                System.out.println("");
                System.out.println(usageNotes);
                throw new ParseException("Incorrect mode option.");
            }
        }
        if (line.hasOption(kernelWidth)) {
            this.sigma = Double.parseDouble(line.getOptionValue(kernelWidth));
        }
        if (line.hasOption(mi)) {
            this.threshold = Double.parseDouble(line.getOptionValue(mi));
        }
        if (line.hasOption(pvalue)) {
            this.pvalue = Double.parseDouble(line.getOptionValue(pvalue));
        }
        if (line.hasOption(tolerance)) {
            this.eps = Double.parseDouble(line.getOptionValue(tolerance));
        }
        if (line.hasOption(hub)) {
            this.hub = line.getOptionValue(hub);
        }
        if (line.hasOption(resample)) {
            this.sample = Integer.parseInt(line.getOptionValue(resample));
        }
        if (line.hasOption(subnet)) {
            this.subnetfile = line.getOptionValue(subnet);
        }
        if (line.hasOption(transfac)) {
            this.annotfile = line.getOptionValue(transfac);
        }
        if (line.hasOption(reconstruct)) {
            String optionValue = line.getOptionValue(reconstruct);
            this.condition = optionValue.substring(0, 1);
            this.controlId = optionValue.substring(1, optionValue.length() - 1);
        }
        if (line.hasOption(filter)) {
            String[] optionValues = line.getOptionValues(filter);
            this.mean = Double.parseDouble(optionValues[0]);
            this.cv = Double.parseDouble(optionValues[1]);
        }
        if (line.hasOption(thresholdFile)) {
            this.threshold_file = line.getOptionValue(thresholdFile);
        }
        if (line.hasOption(kernelFile)) {
            this.kernel_file = line.getOptionValue(kernelFile);
        }
    }

    /* read the list of nodes to be included in constructing a subnetwork from a
    * file.
    */

    static int readProbeList(String infile, Vector<String> probeList) {
        int count = 0;
        try {
            BufferedReader reader = new BufferedReader(new FileReader(infile));

            String line = reader.readLine();
            while (line != null) {
                probeList.add(line);
                count++;
                line = reader.readLine();
            }
        } catch (IOException e) {
            System.err.println("Problem reading probe list file " + infile);
            e.printStackTrace();
            return -1;
        }
        return count;
    }

    public String getAdjfile() {
        return adjfile;
    }

    public void setAdjfile(String adjfile) {
        this.adjfile = adjfile;
    }

    public ALGORITHM getAlgorithm() {
        return algorithm;
    }

    public void setAlgorithm(ALGORITHM algorithm) {
        this.algorithm = algorithm;
    }

    public String getAnnotfile() {
        return annotfile;
    }

    public void setAnnotfile(String annotfile) {
        this.annotfile = annotfile;
    }

    public String getCondition() {
        return condition;
    }

    public void setCondition(String condition) {
        this.condition = condition;
    }

    public String getControlId() {
        return controlId;
    }

    public void setControlId(String controlId) {
        this.controlId = controlId;
    }

    public double getCorrection() {
        return correction;
    }

    public void setCorrection(double correction) {
        this.correction = correction;
    }

    public double getCv() {
        return cv;
    }

    public void setCv(double cv) {
        this.cv = cv;
    }

    /**
     * Also known as the DPI threshold.
     */
    public double getEps() {
        return eps;
    }

    /**
     * Also known as the DPI threshold.
     * @param eps
     */
    public void setEps(double eps) {
        this.eps = eps;
    }

    public String getHome_dir() {
        return home_dir;
    }

    public void setHome_dir(String home_dir) {
        this.home_dir = home_dir;
    }

    public String getHub() {
        return hub;
    }

    public void setHub(String hub) {
        this.hub = hub;
    }

    public String getInfile() {
        return infile;
    }

    public void setInfile(String infile) {
        this.infile = infile;
    }

    public double getMean() {
        return mean;
    }

    public void setMean(double mean) {
        this.mean = mean;
    }

    public MODE getMode() {
        return mode;
    }
    
    public void setMode(MODE mode) {
        this.mode = mode;
    }
    
    public int getMiSteps() {
        return miSteps;
    }

    public void setMiSteps(int miSteps) {
        this.miSteps = miSteps;
    }

    public String getOutfile() {
        return outfile;
    }

    public void setOutfile(String outfile) {
        this.outfile = outfile;
    }

    public double getPercent() {
        return percent;
    }

    public void setPercent(double percent) {
        this.percent = percent;
    }

    public double getPvalue() {
        return pvalue;
    }

    public void setPvalue(double pvalue) {
        this.pvalue = pvalue;
    }

    public int getSample() {
        return sample;
    }

    public void setSample(int sample) {
        this.sample = sample;
    }

    public double getSigma() {
        return sigma;
    }

    public void setSigma(double sigma) {
        this.sigma = sigma;
    }

    /**
     * MI threshold
     * @return
     */
    public double getThreshold() {
        return threshold;
    }

    /**
     * MI threshold
     * @param threshold
     */
    public void setThreshold(double threshold) {
        this.threshold = threshold;
    }

    public String getSubnetfile() {
        return subnetfile;
    }

    public void setSubnetfile(String subnetfile) {
        this.subnetfile = subnetfile;
    }

    public Vector<String> getSubnet() {
        return subnet;
    }

    public void setSubnet(Vector<String> subnet) {
        this.subnet = subnet;
    }

    /**
     * Get the list of gene names that are set as transcription factors.
     * @return
     */
    public Vector<String> getTf_list() {
        return tf_list;
    }

    /**
     * Set the list of gene names that are transcription factors.
     * @param tf_list
     */
    public void setTf_list(Vector<String> tf_list) {
        this.tf_list = tf_list;
    }

    public boolean isSuppressFileWriting() {
        return suppressFileWriting;
    }

    public void setSuppressFileWriting(boolean suppressFileWriting) {
        this.suppressFileWriting = suppressFileWriting;
    }

    public WeightedGraph getPrecomputedAdjacencies() {
        return precomputedAdjacencies;
    }

    /**
     * Use this as a programmatic alternative to adjFile when you already have the adjacency matrix you want to perform
     * DPI on in memory.
     * @param precomputedAdjacencies
     */
    public void setPrecomputedAdjacencies(WeightedGraph precomputedAdjacencies) {
        this.precomputedAdjacencies = precomputedAdjacencies;
    }

    private static String usageNotes = "[*]\tIf no output file is specified by the user, an output will be automatically generated in \n" +
            "\tthe same directory as the input file by appending some of the parameter values, such as\n" +
            "\tkernel width, MI threshold, tolerance and so on, at the end of the input file name, and\n" +
            "\tchanging the file extension to \".adj\".\n" +
            "\n" +
            "[**]\tIf the \"-t\" option is supplied, it will enforce the program to use the specified MI \n" +
            "\tthreshold, therefore the \"-p\" option will be ignored. Otherwise, the program will \n" +
            "\tautomatically determines the MI threshold given the p-value. The default, p-value=1, will\n" +
            "\tpreserve all pairwise MI. \n" +
            "\n" +
            "[***]\tThis option is ideal for transcriptional network reconstruction. If provided, DPI will not\n" +
            "\tremove any connection of a transcription factor (TF) by connections between two probes not\n" +
            "\tannotated as TFs. This option is often used in conjunction with '-s', which specifies a list\n" +
            "\tof probes that are either the same or a subset of the probes specified by '-l'.  \t\n" +
            "\n" +
            "\n" +
            "[****]\tConditional network reconstructs the network given a specified probe being most expressed or \n" +
            "\tleast expressed. In the format that follows \"-c\", \"probeId\" indicate the probe to be \n" +
            "\tconditioned on; \"+\" or \"-\" specify whether the upper or lower tail of the probe's expression\n" +
            "\tshould be used as the condition, and \"%\" is a percentage between (0, 1) specifying the \n" +
            "\tproportion of samples used as the conditioning subset. \n" +
            "\tExample useage:\"-c +24 0.35\", \"-c -1973_s_at 0.4\". \t";

}
