package aracne;

import edu.columbia.c2b2.aracne.Aracne;
import edu.columbia.c2b2.aracne.Matrix;
import edu.columbia.c2b2.aracne.Parameter;
import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import wb.data.MicroarraySet;
import wb.engine.export.TabDelimitedTextOutput;
import wb.plugins.aracne.GraphEdge;
import wb.plugins.aracne.WeightedGraph;
import wb.test.util.TestData;

import java.io.ByteArrayOutputStream;
import java.util.List;

/**
 * Test suite for ARACNE C++ port.
 * User: mhall
 * Date: Aug 30, 2006
 * Time: 11:49:22 AM
 */
public class AracneTest extends TestCase {

    static Log log = LogFactory.getLog(AracneTest.class);

    public void testGraphConversion() throws Exception {
        WeightedGraph weightedGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Matrix matrix = new Matrix(microarraySet, weightedGraph);
        assertEquals("Node count should be equal.", weightedGraph.getNodes().size(), matrix.size());
        WeightedGraph convertedGraph = matrix.getWeightedGraph(false);
        convertedGraph.setName(weightedGraph.getName());
        assertEquals("Node count should be equal.", weightedGraph.getNodes().size(), convertedGraph.getNodes().size());

        assertTrue("Graphs should be equal.", basicCompare(weightedGraph, convertedGraph));
        assertTrue("Graphs should be equal.", basicCompare(weightedGraph, matrix.getWeightedGraph(microarraySet, false)));
    }

    private void logGraph(WeightedGraph convertedGraph) throws Exception {
        ByteArrayOutputStream outstream = new ByteArrayOutputStream();
        TabDelimitedTextOutput out = new TabDelimitedTextOutput(outstream);
        convertedGraph.exportTabular(out);
        out.finishWriting();
        log.debug(outstream.toString());
    }

    public void testSameAsC10x336NoParams() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336kernel05() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.5.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setSigma(0.5);
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336threshold05() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13_t0.5.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setThreshold(0.5);
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336pvalue001() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13_t0.03.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setPvalue(0.001);
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336DPI01() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13_e0.1.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setEps(0.1);
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336HubG1() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_hG1_k0.13.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setHub("G1");
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336Fast() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13-fast.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setAlgorithm(Parameter.ALGORITHM_FAST);
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336Filtered() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13-filtered.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setMean(20);
        p.setCv(1);
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336Conditional() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_cG1H_k0.153.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setCondition("+");
        p.setControlId("G1");
        p.setPercent(0.5);
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336Subnet() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13-subnet.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setSubnetfile("testdata/aracne/probeidlist.txt");
        p.setSuppressFileWriting(true);
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336Transfac() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13-transfac.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setSuppressFileWriting(true);
        p.setAnnotfile("testdata/aracne/probeidlist.txt");
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336ExistingMatrixThreshold() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13_e0.1-post0.5filter.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setSuppressFileWriting(true);
        p.setAdjfile("testdata/aracne/arraydata10x336_k0.5.adj");
        p.setEps(0.1);
        p.setOutfile("testout.adj");
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC10x336ExistingMatrixThresholdOverride() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.13_e0.1-post0.5filter.adj");
        WeightedGraph precomputedGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata10x336_k0.5.adj");
        MicroarraySet microarraySet = loadTestDataSet();
        Parameter p = new Parameter();
        p.setSuppressFileWriting(true);
//        p.setAdjfile("testdata/aracne/arraydata10x336_k0.5.adj");
        p.setPrecomputedAdjacencies(precomputedGraph);
        p.setEps(0.1);
        p.setOutfile("testout.adj");
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }

    public void testSameAsC100x336TransfacSubnetsAndThresholds() throws Exception {
        WeightedGraph origGraph = TestData.loadWeightedGraph("testdata/aracne/arraydata100x336_k0.164_t0.13_e0.adj");
        MicroarraySet microarraySet = TestData.loadTestMicroarraySet("testdata/aracne/arraydata100x336.exp");
        Parameter p = new Parameter();
        p.setSuppressFileWriting(true);
        p.setSigma(0.1636);
        p.setThreshold(0.1335);
        p.setEps(0);
        p.setAnnotfile("testdata/aracne/transfac_probe.txt");
        p.setSubnetfile("testdata/aracne/transfac_probe.txt");
        WeightedGraph weightedGraph = Aracne.run(microarraySet, p);
        log.debug("--- Original graph");
        logGraph(origGraph);
        log.debug("--- Generated Graph");
        logGraph(weightedGraph);

        assertTrue(basicCompare(origGraph, weightedGraph));
    }


    private MicroarraySet loadTestDataSet() throws Exception {
        return TestData.loadTestMicroarraySet("testdata/aracne/arraydata10x336.exp");
    }


    private boolean basicCompare(WeightedGraph graph1, WeightedGraph graph2) {
        if (graph1.getNodes().size() != graph2.getNodes().size()) {
            log.debug("Node counts didn't match.");
            return false;
        }
        if (graph1.getEdges().size() != graph2.getEdges().size()) {
            log.debug("Edge counts don't match.");
            return false;
        }
        for (String node1 : graph1.getNodes()) {
            List<GraphEdge> edgesForNode1 = graph1.getEdgesForNode(node1);
            List<GraphEdge> edgesForNode2 = graph2.getEdgesForNode(node1);
            for (GraphEdge graphEdge : edgesForNode1) {
                boolean found = false;
                for (GraphEdge edge : edgesForNode2) {
                    if (graphEdge.lenientEquals(edge, 10e-7f)) {
                        found = true;
                        break;
                    }
                }
                if (!found) {
                    log.debug("Graph 2 missing edge "+graphEdge.getNode1()+" > "+graphEdge.getNode2()+ " ("+graphEdge.getWeight()+")");
                    return false;
                }
            }
        }
        return true;
    }
}
